/* ARITH_N.C ************************************************************** 
** ALang Execution/Mutation Engine: Version 0.61
** Part of Project von Neumann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Neumann.
**
** License is granted for the use of this software only as part of
** Project von Neumann.
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "stack.h"
#include "list.h"
#include "exec.h"
#include "gene.h"
#include "nodetype.h"
#include "tab.h"
#include "macros.h"

/******************************* PLUS *******************************
** Sums its (int) children.
*/

void PLUS_exe() {

  if (STATUS > 0)
    IRETVAL += IPOP();

  if (STATUS < NUMBER(THISNODE)) {
    ENTER_NODE(CHILD(THISNODE, STATUS++));
  }
  EXIT_NODE();
}


/******************************* MULT *******************************
** Multiplies its (int) children.
*/

void MULT_exe() {

  if (STATUS > 0)
    IRETVAL *= IPOP();
  else
    IRETVAL = 1;

  if (STATUS < NUMBER(THISNODE)) {
    ENTER_NODE(CHILD(THISNODE, STATUS++));
  }
  EXIT_NODE();
}


/******************************* SUBT ******************************
** Subtracts its (int) second child from the first.
*/

void SUBT_exe() {
  switch (STATUS++) {
      case 0:
      ENTER_NODE(CHILD(THISNODE, 0));

    case 1:
      ENTER_NODE(CHILD(THISNODE, 1));

    case 2:
      IRETVAL = IARG(0) - IARG(1);
      EXIT_NODE();
  }
}


/******************************* DIV ******************************
** Subtracts its (int) second child from the first.
*/

void DIV_exe() {
  switch (STATUS++) {
      case 0:
      ENTER_NODE(CHILD(THISNODE, 0));

    case 1:
      ENTER_NODE(CHILD(THISNODE, 1));

    case 2:
      if (IARG(1) != 0) {
        IRETVAL = IARG(0) / IARG(1);
      } else {
        IRETVAL = 0;
      }
      EXIT_NODE();
  }
}



/******************************* AND *******************************
** Keeps executing its children until one of them returns zero.
** Returns 1 only if they all return non-zero.
*/

void AND_exe() {
  if (STATUS > 0) {
    IRETVAL = IPOP();
    if (IRETVAL == 0) {
      /* IRETVAL already zero */
      EXIT_NODE();
    }
  }
  if (STATUS < NUMBER(THISNODE)) {
    ENTER_NODE(CHILD(THISNODE, STATUS++));
  }
  IRETVAL = 1;
  EXIT_NODE();
}
/******************************* OR *******************************
** Keeps executing its children until one of them returns non-zero.
** Returns 0 if they all return zero.
*/

void OR_exe() {

  if (STATUS > 0) {
    IRETVAL = IPOP();
    if (IRETVAL) {
      IRETVAL = 1;
      EXIT_NODE();
    }
  }
  if (STATUS < NUMBER(THISNODE)) {
    ENTER_NODE(CHILD(THISNODE, STATUS++));
  }
  /* IRETVAL still zero */
  EXIT_NODE();
}
/******************************* XOR *******************************
** Executes all its children in turn. Returns 1 if an odd number
** retuned non-zero, otherwise zero.
*/

void XOR_exe() {

  if (STATUS && IPOP()) {
    IRETVAL ^= 1;
  }
  if (STATUS < NUMBER(THISNODE)) {
    ENTER_NODE(CHILD(THISNODE, STATUS++));
  }
  EXIT_NODE();
}
