/* MISC_N.C ************************************************************** 
** ALang Execution/Mutation Engine: Version 0.61
** Part of Project von Neumann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Neumann.
**
** License is granted for the use of this software only as part of
** Project von Neumann.
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "list.h"
#include "stack.h"
#include "exec.h"
#include "nodetype.h"
#include "tab.h"
#include "macros.h"

/* #include "vars.h" */

/****************************** PRINT *******************************
** prints the (int/obj) value of its only child and returns it.
*/

void PRI_exe(struct AL_Node * where) {
  if (!STATUS++) {
    ENTER_NODE(CHILD(THISNODE, 0));
  } else {
    printf("print : " EINT32_PRINTF_FORMAT "\n", IARG(0));
    IRETVAL = IARG(0);
    EXIT_NODE();
  }
}

void PRI_res(AL_Attrib ret_type,
             struct AL_Node * where,
             int i,
             AL_Attrib buffer) {
/* note: this node's return-type and child#0 type are interdependant
** this could be a problem because the return type supplied might not
** be one that can legitimately specify the child (e.g in this case
** both must be SING_ATTR | VAL_ATTR | RET_ATTR).  To get around this,
** we take the return value and check it, if it doesn't fit we return
** ERROR_ATTR which the calling routine must take as
** "cannot supply that return type". */
  if (!ACCEPTS_ATTR(ret_type[0], SING_ATTR | RET_ATTR | VAL_ATTR)) {
    buffer[0] = ERROR_ATTR;
    return;
  }
  switch (i) {
    case 0:
      COPY_ATTR(buffer, ret_type);
      LOSE_ATTR(buffer[0], MULT_ATTR | REF_ATTR | FLAV_ATTR);
      ADD_ATTR(buffer[0], SING_ATTR | VAL_ATTR | RET_ATTR);
      break;
    case 1:
      buffer[0] = END_ATTR;
      break;
    default:
      buffer[0] = ERROR_ATTR;
  }
}

void PRI_attr(struct AL_Node *where, AL_Attrib buffer) {
  ATTR(CHILD(where, 0), buffer);
}

/****************************** LPRINT ******************************
** prints the (list) value of its only child and returns it.
*/

void LPRI_exe(struct AL_Node * where) {
  if (!STATUS++) {
    ENTER_NODE(CHILD(THISNODE, 0));
  } else {
    int i;
    struct AL_ListEntry *entry;
    
    for(entry = ((struct AL_ListHead *)IARG(0))->first, i = 0;
        entry;
        entry = entry->next, i++) {
      printf("print : (%3.3i) : " EINT32_PRINTF_FORMAT "\n",
             i, entry->value);
    }
    IRETVAL = IARG(0);
    EXIT_NODE();
  }
}

void LPRI_res(AL_Attrib ret_type,
             struct AL_Node * where,
             int i,
             AL_Attrib buffer) {
  if (!ACCEPTS_ATTR(ret_type[0], SING_ATTR | RET_ATTR | VAL_ATTR)) {
    buffer[0] = ERROR_ATTR;
    return;
  }
  switch (i) {
    case 0:
      COPY_ATTR(buffer, ret_type);
      LOSE_ATTR(buffer[0], MULT_ATTR | REF_ATTR | FLAV_ATTR);
      ADD_ATTR(buffer[0], SING_ATTR | VAL_ATTR | RET_ATTR);
      break;
    case 1:
      buffer[0] = END_ATTR;
      break;
    default:
      buffer[0] = ERROR_ATTR;
  }
}

void LPRI_attr(struct AL_Node *where, AL_Attrib buffer) {
  ATTR(CHILD(where, 0), buffer);
}
